/*
Copyright © 2023 Philipp Wolfer <phw@uploadedlobster.com>

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/
package models

import (
	"strings"
	"time"
)

type MBID string

type AdditionalInfo map[string]any

type Track struct {
	TrackName        string
	ReleaseName      string
	ArtistNames      []string
	TrackNumber      int
	Duration         time.Duration
	Isrc             string
	RecordingMbid    MBID
	ReleaseMbid      MBID
	ReleaseGroupMbid MBID
	ArtistMbids      []MBID
	WorkMbids        []MBID
	Tags             []string
	AdditionalInfo   AdditionalInfo
}

func (t Track) ArtistName() string {
	return strings.Join(t.ArtistNames, ", ")
}

type Listen struct {
	Track
	ListenedAt       time.Time
	PlaybackDuration time.Duration
	UserName         string
}

type Love struct {
	Track
	Created       time.Time
	UserName      string
	RecordingMbid MBID
	RecordingMsid MBID
}
